/*!
 * @comet-cli/adapter-openapi v0.1.5
 * (c) Elisha Witte <elishawitte@gmail.com>
 * Released under the MIT License.
 */
'use strict';

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = o[Symbol.iterator]();
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var parser = require('swagger-parser');

var converter = require('swagger2openapi');

var Parser = /*#__PURE__*/function () {
  function Parser() {
    _classCallCheck(this, Parser);
  }

  _createClass(Parser, null, [{
    key: "load",
    value: function load(pathOrObject) {
      try {
        var _this2 = this;

        return _await(parser.bundle(pathOrObject, {
          resolve: {
            http: {
              withCredentials: false
            }
          }
        }), function (spec) {
          return spec.swagger !== undefined ? _this2.convertSwaggerToOpenApi(spec) : spec;
        });
      } catch (e) {
        return Promise.reject(e);
      }
    }
  }, {
    key: "convertSwaggerToOpenApi",
    value: function convertSwaggerToOpenApi(spec) {
      return new Promise(function (resolve, reject) {
        converter.convertObj(spec, {
          patch: true,
          warnOnly: true
        }, function (err, res) {
          if (err) {
            return reject(err);
          }

          resolve(res && res.openapi);
        });
      });
    }
  }]);

  return Parser;
}();

var InformationTransformer = /*#__PURE__*/function () {
  function InformationTransformer() {
    _classCallCheck(this, InformationTransformer);
  }

  _createClass(InformationTransformer, null, [{
    key: "execute",
    value: function execute(spec) {
      var name = spec.entity.info.title;
      var description = spec.entity.info.description;
      var version = spec.entity.info.version;
      var license = spec.entity.info.license;
      var terms = spec.entity.info.termsOfService;
      var contact = spec.entity.info.contact;
      var links = '';

      if (contact !== undefined) {
        var contactLinks = '';

        if (contact.name) {
          contactLinks += "".concat(contact.name, ": ");
        }

        if (contact.email) {
          contactLinks += "[".concat(contact.email, "](mailto:").concat(contact.email, ") - ");
        }

        if (contact.url) {
          contactLinks += "[".concat(contact.url, "](").concat(contact.url, ")");
        }

        links += contactLinks;
      }

      if (license !== undefined) {
        links += " | License: [".concat(license.name, "](").concat(license.url, ")");
      }

      if (terms !== undefined) {
        links += " | [Terms of Service](".concat(terms, ")");
      }

      if (spec.entity.externalDocs) {
        if (links !== '') {
          links += '\n';
        }

        links += "[".concat(spec.entity.externalDocs.description, "](").concat(spec.entity.externalDocs.url, ")");
      }

      if (links !== '') {
        description = "".concat(links, "\n\n").concat(description);
      }

      return {
        name: name,
        description: description,
        version: version,
        servers: spec.entity.servers ? this.getServers(spec) : []
      };
    }
  }, {
    key: "getServers",
    value: function getServers(spec) {
      var servers = [];

      var _iterator = _createForOfIteratorHelper(spec.entity.servers || []),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var server = _step.value;
          var variables = {};

          if (server.variables) {
            for (var _i = 0, _Object$keys = Object.keys(server.variables); _i < _Object$keys.length; _i++) {
              var name = _Object$keys[_i];
              variables[name] = {
                "default": server.variables[name]["default"],
                description: server.variables[name].description,
                "enum": server.variables[name]["enum"]
              };
            }
          }

          servers.push({
            uri: server.url,
            description: server.description || undefined,
            variables: server.variables ? variables : undefined
          });
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return servers;
    }
  }]);

  return InformationTransformer;
}();

var jsonPointer = require('json-pointer');

var origParse = jsonPointer.parse;
var lib = jsonPointer;
var JsonPointer = /*#__PURE__*/function () {
  function JsonPointer() {
    _classCallCheck(this, JsonPointer);
  }

  _createClass(JsonPointer, null, [{
    key: "baseName",
    value: function baseName(pointer) {
      var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      var tokens = JsonPointer.parse(pointer);
      return tokens[tokens.length - level];
    }
  }, {
    key: "dirName",
    value: function dirName(pointer) {
      var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      var tokens = JsonPointer.parse(pointer);
      return jsonPointer.compile(tokens.slice(0, tokens.length - level));
    }
  }, {
    key: "relative",
    value: function relative(from, to) {
      var fromTokens = JsonPointer.parse(from);
      var toTokens = JsonPointer.parse(to);
      return toTokens.slice(fromTokens.length);
    }
  }, {
    key: "parse",
    value: function parse(pointer) {
      var ptr = pointer;

      if (ptr.charAt(0) === '#') {
        ptr = ptr.substring(1);
      }

      return origParse(ptr);
    }
  }, {
    key: "get",
    value: function get(object, pointer) {
      return jsonPointer.get(object, pointer);
    }
  }]);

  return JsonPointer;
}();
lib.parse = JsonPointer.parse;
Object.assign(JsonPointer, lib);

var RefCounter = /*#__PURE__*/function () {
  function RefCounter() {
    _classCallCheck(this, RefCounter);

    this.counter = {};
  }

  _createClass(RefCounter, [{
    key: "visit",
    value: function visit(ref) {
      this.counter[ref] = this.counter[ref] ? this.counter[ref] + 1 : 1;
    }
  }, {
    key: "exit",
    value: function exit(ref) {
      this.counter[ref] = this.counter[ref] && this.counter[ref] - 1;
    }
  }, {
    key: "visited",
    value: function visited(ref) {
      return !!this.counter[ref];
    }
  }]);

  return RefCounter;
}();

var Specification = /*#__PURE__*/function () {
  function Specification(spec) {
    _classCallCheck(this, Specification);

    this.refCounter = new RefCounter();
    this.entity = spec;
  }

  _createClass(Specification, [{
    key: "deref",
    value: function deref(obj) {
      var forceCircular = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      if (this.isRef(obj)) {
        var resolved = this.byRef(obj.$ref);
        var visited = this.refCounter.visited(obj.$ref);
        this.refCounter.visit(obj.$ref);

        if (visited && !forceCircular) {
          return Object.assign({}, resolved, {
            'x-circular-ref': true
          });
        }

        if (this.isRef(resolved)) {
          var res = this.deref(resolved);
          this.exitRef(resolved);
          return res;
        }

        return resolved;
      }

      return obj;
    }
  }, {
    key: "isRef",
    value: function isRef(obj) {
      if (!obj) {
        return false;
      }

      return obj.$ref !== undefined && obj.$ref !== null;
    }
  }, {
    key: "byRef",
    value: function byRef(ref) {
      var res;
      var reference = ref;

      if (!this.entity) {
        return;
      }

      if (reference.charAt(0) !== '#') {
        reference = "#".concat(reference);
      }

      reference = decodeURIComponent(reference);

      try {
        res = JsonPointer.get(this.entity, reference);
      } catch (e) {}

      return res || {};
    }
  }, {
    key: "resetVisited",
    value: function resetVisited() {
      this.refCounter = new RefCounter();
    }
  }, {
    key: "exitRef",
    value: function exitRef(ref) {
      if (!this.isRef(ref)) {
        return;
      }

      this.refCounter.exit(ref.$ref);
    }
  }, {
    key: "exitParents",
    value: function exitParents(schema) {
      var _iterator = _createForOfIteratorHelper(schema.parentRefs || []),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var parent$ref = _step.value;
          this.exitRef({
            $ref: parent$ref
          });
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
    }
  }]);

  return Specification;
}();

var AuthenticationTransformer = /*#__PURE__*/function () {
  function AuthenticationTransformer() {
    _classCallCheck(this, AuthenticationTransformer);
  }

  _createClass(AuthenticationTransformer, null, [{
    key: "execute",
    value: function execute(spec) {
      var schemes = spec.entity.components && spec.entity.components.securitySchemes || {};
      var auth = {};

      for (var _i = 0, _Object$keys = Object.keys(schemes); _i < _Object$keys.length; _i++) {
        var name = _Object$keys[_i];
        var scheme = spec.deref(schemes[name]);
        var type = 'key';

        switch (scheme.type) {
          case 'http':
            if (scheme.scheme === 'basic' || scheme.scheme === 'digest') {
              type = scheme.scheme;
            }

            if (scheme.bearerFormat === 'JWT') {
              type = 'jwt';
            }

            break;

          case 'apiKey':
            type = 'key';
            break;

          case 'oauth2':
            type = 'oauth2';
        }

        var flows = {};

        if (scheme.flows) {
          for (var _i2 = 0, _Object$keys2 = Object.keys(scheme.flows); _i2 < _Object$keys2.length; _i2++) {
            var flowType = _Object$keys2[_i2];
            var flow = scheme.flows[flowType];
            flows[flowType] = {
              refreshUri: flow.refreshUrl,
              authorizationUri: flow.authorizationUrl,
              tokenUri: flow.tokenUrl,
              scopes: flow.scopes
            };
          }
        }

        var description = scheme.description || '';

        if (scheme.bearerFormat) {
          description = "\n Bearer-Format: ".concat(scheme.bearerFormat);
        }

        auth[name] = {
          type: type,
          flows: flows,
          description: description || undefined,
          name: scheme.name,
          location: scheme["in"]
        };
      }

      return auth;
    }
  }]);

  return AuthenticationTransformer;
}();

var sampler = require('openapi-sampler');

var SchemaValueResolver = /*#__PURE__*/function () {
  function SchemaValueResolver() {
    _classCallCheck(this, SchemaValueResolver);
  }

  _createClass(SchemaValueResolver, null, [{
    key: "execute",
    value: function execute(schema) {
      var value;
      value = this.resolveExampleValue(schema);

      if (value !== undefined) {
        return value;
      }

      value = this.resolveDefaultValue(schema);

      if (value !== undefined) {
        return value;
      }

      value = this.resolveEnumValue(schema);

      if (value !== undefined) {
        return value;
      }

      if (schema.type && schema.type === 'boolean') {
        return true;
      }

      if (schema.type && schema.type === 'object') {
        return this.resolveObjectValue(schema);
      }

      if (schema.type && schema.type === 'array' && schema.items && schema.items.type === 'object') {
        return [this.resolveObjectValue(schema.items)];
      }

      return sampler.sample(schema);
    }
  }, {
    key: "resolveObjectValue",
    value: function resolveObjectValue(schema) {
      if (schema.properties === undefined) {
        return undefined;
      }

      var value = {};

      for (var _i = 0, _Object$keys = Object.keys(schema.properties); _i < _Object$keys.length; _i++) {
        var name = _Object$keys[_i];
        var result = this.execute(schema.properties[name]);

        if (result !== undefined) {
          value[name] = result;
        } else if (schema.required && schema.required.includes(name)) {
          value[name] = "<".concat(schema.properties[name].type, ">");
        }
      }

      return value;
    }
  }, {
    key: "resolveExampleValue",
    value: function resolveExampleValue(schema) {
      if (schema.hasOwnProperty('example')) {
        return schema.example;
      }

      return undefined;
    }
  }, {
    key: "resolveDefaultValue",
    value: function resolveDefaultValue(schema) {
      if (schema.hasOwnProperty('default')) {
        return schema["default"];
      }

      if (schema.items && schema.items.hasOwnProperty('default')) {
        return [schema.items["default"]];
      }

      return undefined;
    }
  }, {
    key: "resolveEnumValue",
    value: function resolveEnumValue(schema) {
      if (schema["enum"]) {
        return schema["enum"][Math.floor(Math.random() * schema["enum"].length)];
      }

      if (schema.items && schema.items["enum"]) {
        return [schema.items["enum"][Math.floor(Math.random() * schema.items["enum"].length)]];
      }

      return undefined;
    }
  }]);

  return SchemaValueResolver;
}();

var SchemaTransformer = /*#__PURE__*/function () {
  function SchemaTransformer() {
    _classCallCheck(this, SchemaTransformer);
  }

  _createClass(SchemaTransformer, null, [{
    key: "execute",
    value: function execute(spec, ref) {
      var type = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'other';
      var $ref = ref.$ref;
      var rawSchema = spec.deref(ref);
      rawSchema = this.mergeAllOf(spec, rawSchema, $ref);
      var schema = {};
      var isCircular = !!rawSchema['x-circular-ref'];

      if (isCircular) {
        schema['x-circular-ref'] = true;
        spec.exitRef(ref);
        return schema;
      }

      var _iterator = _createForOfIteratorHelper(this.getCompatibleProperties()),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var _property = _step.value;
          schema[_property] = rawSchema[_property];
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      if (rawSchema.type !== undefined) {
        schema.type = this.transformType(rawSchema);
      }

      if (Array.isArray(rawSchema["enum"])) {
        schema["enum"] = this.transformEnum(rawSchema);
      }

      if (rawSchema.discriminator) {
        schema.discriminator = this.transformDiscriminator(rawSchema);
      }

      var nested = ['allOf', 'oneOf', 'anyOf', 'not', 'items', 'additionalProperties'];

      for (var _i = 0, _nested = nested; _i < _nested.length; _i++) {
        var struct = _nested[_i];

        if (rawSchema[struct] === undefined) {
          continue;
        }

        if (Array.isArray(rawSchema[struct])) {
          schema[struct] = [];

          for (var i = 0; i < rawSchema[struct].length; i = i + 1) {
            schema[struct][i] = this.execute(spec, rawSchema[struct][i], type);
          }
        } else if (_typeof(rawSchema[struct]) === 'object') {
          schema[struct] = this.execute(spec, rawSchema[struct], type);
        }
      }

      if (rawSchema.properties !== undefined && _typeof(rawSchema.properties) === 'object') {
        schema.properties = {};

        for (var _i2 = 0, _Object$keys = Object.keys(rawSchema.properties); _i2 < _Object$keys.length; _i2++) {
          var name = _Object$keys[_i2];
          var property = rawSchema.properties[name];

          if (type === 'request' && property.readOnly || type === 'response' && property.writeOnly) {
            delete rawSchema.properties[name];
          } else {
            schema.properties[name] = this.execute(spec, rawSchema.properties[name], type);
          }
        }
      }

      spec.exitRef(ref);
      spec.exitParents(rawSchema);
      return schema;
    }
  }, {
    key: "transformType",
    value: function transformType(schema) {
      var type = this.detectType(schema);

      if (typeof type === 'string' && schema.nullable === true) {
        return [type, 'null'];
      }

      return type;
    }
  }, {
    key: "transformEnum",
    value: function transformEnum(schema) {
      if (schema["enum"] && typeof schema.type === 'string' && schema.nullable === true) {
        return schema["enum"].concat([null]);
      }

      return schema["enum"];
    }
  }, {
    key: "transformDiscriminator",
    value: function transformDiscriminator(schema) {
      if (schema.discriminator && schema.discriminator.propertyName) {
        return schema.discriminator.propertyName;
      }
    }
  }, {
    key: "detectType",
    value: function detectType(schema) {
      if (schema.type !== undefined) {
        return schema.type;
      }

      var keywords = this.getTypeKeywords();

      for (var _i3 = 0, _Object$keys2 = Object.keys(keywords); _i3 < _Object$keys2.length; _i3++) {
        var keyword = _Object$keys2[_i3];
        var type = keywords[keyword];

        if (schema[keyword] !== undefined) {
          return type;
        }
      }
    }
  }, {
    key: "mergeAllOf",
    value: function mergeAllOf(spec, schema, $ref) {
      var _this = this;

      var allOf = schema.allOf;

      if (allOf === undefined) {
        return schema;
      }

      var merged = Object.assign({}, schema, {
        allOf: undefined,
        parentRefs: []
      });
      var allOfSchemas = allOf.map(function (subSchema) {
        var _merged$parentRefs;

        var resolved = spec.deref(subSchema);
        var subRef = subSchema.$ref || undefined;

        var subMerged = _this.mergeAllOf(spec, resolved, subRef);

        (_merged$parentRefs = merged.parentRefs).push.apply(_merged$parentRefs, _toConsumableArray(subMerged.parentRefs || []));

        return {
          $ref: subRef,
          schema: subMerged
        };
      });

      var _iterator2 = _createForOfIteratorHelper(allOfSchemas),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _step2$value = _step2.value,
              subSchemaRef = _step2$value.$ref,
              subSchema = _step2$value.schema;

          if (merged.type !== subSchema.type && merged.type !== undefined && subSchema.type !== undefined) {
            throw new Error("Incompatible types in allOf at \"".concat($ref, "\""));
          }

          if (subSchema.type !== undefined) {
            merged.type = subSchema.type;
          }

          if (subSchema.properties !== undefined) {
            merged.properties = merged.properties || {};

            for (var _i4 = 0, _Object$keys3 = Object.keys(subSchema.properties); _i4 < _Object$keys3.length; _i4++) {
              var key = _Object$keys3[_i4];

              if (!merged.properties[key]) {
                merged.properties[key] = subSchema.properties[key];
              } else {
                merged.properties[key] = this.mergeAllOf(spec, {
                  allOf: [merged.properties[key], subSchema.properties[key]]
                }, "".concat($ref, "/properties/").concat(key));
              }
            }
          }

          if (subSchema.items !== undefined) {
            merged.items = merged.items || {};
            merged.items = this.mergeAllOf(spec, {
              allOf: [merged.items, subSchema.items]
            }, "".concat($ref, "/items"));
          }

          if (subSchema.required !== undefined) {
            merged.required = _toConsumableArray(new Set((merged.required || []).concat(subSchema.required)));
          }

          merged = Object.assign({}, subSchema, merged);

          if (subSchemaRef) {
            merged.parentRefs.push(subSchemaRef);
          }
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      return merged;
    }
  }, {
    key: "getCompatibleProperties",
    value: function getCompatibleProperties() {
      return ['description', 'default', 'required', 'format', 'title', 'multipleOf', 'maximum', 'exclusiveMaximum', 'minimum', 'exclusiveMinimum', 'maxLength', 'minLength', 'pattern', 'maxItems', 'minItems', 'uniqueItems', 'maxProperties', 'minProperties', 'xml', 'x-circular-ref', 'example'];
    }
  }, {
    key: "getTypeKeywords",
    value: function getTypeKeywords() {
      return {
        multipleOf: 'number',
        maximum: 'number',
        exclusiveMaximum: 'number',
        minimum: 'number',
        exclusiveMinimum: 'number',
        maxLength: 'string',
        minLength: 'string',
        pattern: 'string',
        items: 'array',
        maxItems: 'array',
        minItems: 'array',
        uniqueItems: 'array',
        maxProperties: 'object',
        minProperties: 'object',
        required: 'object',
        additionalProperties: 'object',
        properties: 'object'
      };
    }
  }]);

  return SchemaTransformer;
}();

var ParameterValueResolver = /*#__PURE__*/function () {
  function ParameterValueResolver() {
    _classCallCheck(this, ParameterValueResolver);
  }

  _createClass(ParameterValueResolver, null, [{
    key: "inferValue",
    value: function inferValue(spec, apiParameter) {
      var value;
      var parameter = spec.deref(apiParameter);
      spec.exitRef(apiParameter);
      value = this.inferExampleValue(spec, parameter);

      if (value !== undefined) {
        return value;
      }

      value = this.inferDefaultValue(spec, parameter);

      if (value !== undefined) {
        return value;
      }

      value = this.inferEnumValue(spec, parameter);

      if (value !== undefined) {
        return value;
      }

      return undefined;
    }
  }, {
    key: "inferExampleValue",
    value: function inferExampleValue(spec, apiParameter) {
      if (apiParameter.hasOwnProperty('example')) {
        return apiParameter.example;
      }

      if (apiParameter.hasOwnProperty('examples') && apiParameter.examples) {
        var examples = Object.keys(apiParameter.examples);
        var index = Math.floor(Math.random() * examples.length);
        var example = spec.deref(apiParameter.examples[index]);
        spec.exitRef(apiParameter.examples[index]);
        return example.value;
      }

      if (apiParameter.schema) {
        var schema = SchemaTransformer.execute(spec, apiParameter.schema);
        var result = SchemaValueResolver.resolveExampleValue(schema);

        if (result !== undefined) {
          return result;
        }
      }

      if (apiParameter.content) {
        var content = apiParameter.content[Object.keys(apiParameter.content)[0]];

        if (content.hasOwnProperty('example')) {
          return content.example;
        }

        if (content.hasOwnProperty('examples')) ;

        if (content.schema) {
          var _schema = SchemaTransformer.execute(spec, content.schema);

          var _result = SchemaValueResolver.resolveExampleValue(_schema);

          if (_result !== undefined) {
            return _result;
          }
        }
      }

      return undefined;
    }
  }, {
    key: "inferDefaultValue",
    value: function inferDefaultValue(spec, apiParameter) {
      if (apiParameter.schema) {
        var schema = SchemaTransformer.execute(spec, apiParameter.schema);
        var result = SchemaValueResolver.resolveDefaultValue(schema);

        if (result !== undefined) {
          return result;
        }
      }

      if (apiParameter.content) {
        var content = apiParameter.content[Object.keys(apiParameter.content)[0]];

        if (content.schema) {
          var _schema2 = SchemaTransformer.execute(spec, content.schema);

          var _result2 = SchemaValueResolver.resolveDefaultValue(_schema2);

          if (_result2 !== undefined) {
            return _result2;
          }
        }
      }

      return undefined;
    }
  }, {
    key: "inferEnumValue",
    value: function inferEnumValue(spec, apiParameter) {
      if (apiParameter.schema) {
        var schema = SchemaTransformer.execute(spec, apiParameter.schema);
        var result = SchemaValueResolver.resolveEnumValue(schema);

        if (result !== undefined) {
          return result;
        }
      }

      if (apiParameter.content) {
        var content = apiParameter.content[Object.keys(apiParameter.content)[0]];

        if (content.schema) {
          var _schema3 = SchemaTransformer.execute(spec, content.schema);

          var _result3 = SchemaValueResolver.resolveEnumValue(_schema3);

          if (_result3 !== undefined) {
            return _result3;
          }
        }
      }

      return undefined;
    }
  }]);

  return ParameterValueResolver;
}();

var HeaderTransformer = /*#__PURE__*/function () {
  function HeaderTransformer() {
    _classCallCheck(this, HeaderTransformer);
  }

  _createClass(HeaderTransformer, null, [{
    key: "transformFromParameters",
    value: function transformFromParameters(spec, params) {
      var headers = [];
      var parameters = params.map(function (param) {
        var parameter = spec.deref(param);
        spec.exitRef(param);
        return parameter;
      });
      var headerParams = parameters.filter(function (param) {
        return param["in"] === 'header';
      });

      var _iterator = _createForOfIteratorHelper(headerParams),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var param = _step.value;
          var schema = param.schema;
          headers.push({
            name: param.name,
            description: param.description || undefined,
            deprecated: param.deprecated || false,
            example: ParameterValueResolver.inferValue(spec, param),
            schema: schema ? SchemaTransformer.execute(spec, schema) : undefined,
            required: param.required || false
          });
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return headers;
    }
  }, {
    key: "transformFromHeaders",
    value: function transformFromHeaders(spec, specHeaders) {
      var headers = [];

      for (var _i = 0, _Object$keys = Object.keys(specHeaders); _i < _Object$keys.length; _i++) {
        var name = _Object$keys[_i];
        var header = spec.deref(specHeaders[name]);
        var schema = header.schema;
        headers.push({
          name: name,
          description: header.description || undefined,
          deprecated: header.deprecated || false,
          example: ParameterValueResolver.inferValue(spec, header),
          schema: schema ? SchemaTransformer.execute(spec, schema) : undefined,
          required: header.required || false
        });
        spec.deref(specHeaders[name]);
      }

      return headers;
    }
  }]);

  return HeaderTransformer;
}();

var ParameterTransformer = /*#__PURE__*/function () {
  function ParameterTransformer() {
    _classCallCheck(this, ParameterTransformer);
  }

  _createClass(ParameterTransformer, null, [{
    key: "execute",
    value: function execute(spec, paramRefs) {
      var parameters = [];
      var params = paramRefs.map(function (param) {
        var parameter = spec.deref(param);
        spec.exitRef(param);
        return parameter;
      });

      var _iterator = _createForOfIteratorHelper(params),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var param = _step.value;
          var schema = param.schema;

          if (param["in"] !== 'header') {
            parameters.push({
              name: param.name,
              description: param.description || undefined,
              deprecated: param.deprecated || false,
              location: param["in"],
              required: param.required || false,
              schema: schema ? SchemaTransformer.execute(spec, schema) : undefined,
              example: ParameterValueResolver.inferValue(spec, param)
            });
          }
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return parameters;
    }
  }]);

  return ParameterTransformer;
}();

var BodyResolver = /*#__PURE__*/function () {
  function BodyResolver() {
    _classCallCheck(this, BodyResolver);
  }

  _createClass(BodyResolver, null, [{
    key: "execute",
    value: function execute(spec, contents) {
      var type = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'other';
      var bodies = {};

      if (contents === undefined) {
        return undefined;
      }

      for (var _i = 0, _Object$keys = Object.keys(contents); _i < _Object$keys.length; _i++) {
        var mime = _Object$keys[_i];
        spec.resetVisited();
        var content = contents[mime];
        var requestSchema = content.schema;
        var schema = requestSchema ? SchemaTransformer.execute(spec, requestSchema, type) : undefined;
        bodies[mime] = {
          mediaType: mime,
          schema: schema,
          examples: this.resolveExamples(spec, content, schema)
        };
      }

      return bodies;
    }
  }, {
    key: "resolveExamples",
    value: function resolveExamples(spec, content, schema) {
      var examples = [];

      if (content.hasOwnProperty('example')) {
        examples.push(content.example);
      }

      if (content.examples) {
        for (var _i2 = 0, _Object$keys2 = Object.keys(content.examples); _i2 < _Object$keys2.length; _i2++) {
          var name = _Object$keys2[_i2];
          var example = spec.deref(content.examples[name]);

          if (example.value) {
            examples.push(example.value);
          }

          if (example.externalValue) {
            examples.push(example.externalValue);
          }

          spec.exitRef(content.examples[name]);
        }
      }

      if (schema !== undefined) {
        var result = SchemaValueResolver.execute(schema);

        if (result !== undefined) {
          examples.push(result);
        }
      }

      return examples;
    }
  }]);

  return BodyResolver;
}();

var RequestTransformer = /*#__PURE__*/function () {
  function RequestTransformer() {
    _classCallCheck(this, RequestTransformer);
  }

  _createClass(RequestTransformer, null, [{
    key: "execute",
    value: function execute(spec, requestBody, headers) {
      var request = {
        headers: headers,
        description: undefined,
        body: undefined
      };
      var body = spec.deref(requestBody);

      if (body) {
        request.description = body.description || undefined;
        request.body = BodyResolver.execute(spec, body.content, 'request');
      }

      spec.exitRef(requestBody);
      return request;
    }
  }]);

  return RequestTransformer;
}();

var ResponseTransformer = /*#__PURE__*/function () {
  function ResponseTransformer() {
    _classCallCheck(this, ResponseTransformer);
  }

  _createClass(ResponseTransformer, null, [{
    key: "execute",
    value: function execute(spec, specResponses) {
      var responses = {};

      for (var _i = 0, _Object$keys = Object.keys(specResponses); _i < _Object$keys.length; _i++) {
        var code = _Object$keys[_i];
        var response = specResponses[code];
        var headers = response.headers || null;
        var content = response.content;
        responses[code] = {
          statusCode: code === 'default' ? code : Number(code),
          description: response.description || undefined,
          headers: headers ? HeaderTransformer.transformFromHeaders(spec, headers) : [],
          body: content ? BodyResolver.execute(spec, content, 'response') : undefined
        };
      }

      return responses;
    }
  }]);

  return ResponseTransformer;
}();

var OperationTransformer = /*#__PURE__*/function () {
  function OperationTransformer() {
    _classCallCheck(this, OperationTransformer);
  }

  _createClass(OperationTransformer, null, [{
    key: "execute",
    value: function execute(spec, operation, method, resourceHeaders) {
      var params = operation.parameters || [];
      var operationHeaders = HeaderTransformer.transformFromParameters(spec, params);
      var headers = this.mergeHeaders(resourceHeaders, operationHeaders);
      var description = operation.description || operation.summary || '';

      if (operation.externalDocs) {
        description = "[".concat(operation.externalDocs.description, "](").concat(operation.externalDocs.url, ")\n\n").concat(description);
      }

      var requestBody = operation.requestBody;
      return {
        method: method,
        name: operation.summary || operation.operationId || undefined,
        description: description || undefined,
        deprecated: operation.deprecated || false,
        tags: operation.tags,
        parameters: ParameterTransformer.execute(spec, params),
        securedBy: operation.security || [],
        request: requestBody ? RequestTransformer.execute(spec, requestBody, headers) : undefined,
        responses: ResponseTransformer.execute(spec, operation.responses),
        transactions: []
      };
    }
  }, {
    key: "mergeHeaders",
    value: function mergeHeaders(resourceHeaders, operationHeaders) {
      var mergedHeaders = {};

      var _iterator = _createForOfIteratorHelper(resourceHeaders),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var _header = _step.value;
          mergedHeaders[_header.name] = _header;
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      var _iterator2 = _createForOfIteratorHelper(operationHeaders),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _header2 = _step2.value;
          mergedHeaders[_header2.name] = _header2;
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      var headers = [];

      for (var _i = 0, _Object$keys = Object.keys(mergedHeaders); _i < _Object$keys.length; _i++) {
        var header = _Object$keys[_i];
        headers.push(mergedHeaders[header]);
      }

      return headers;
    }
  }]);

  return OperationTransformer;
}();

var ResourceTransformer = /*#__PURE__*/function () {
  function ResourceTransformer() {
    _classCallCheck(this, ResourceTransformer);
  }

  _createClass(ResourceTransformer, null, [{
    key: "execute",
    value: function execute(spec) {
      var resources = [];

      for (var _i = 0, _Object$keys = Object.keys(spec.entity.paths); _i < _Object$keys.length; _i++) {
        var path = _Object$keys[_i];
        var openApiResource = spec.entity.paths[path];
        var params = openApiResource.parameters || [];
        var resourceHeaders = HeaderTransformer.transformFromParameters(spec, params);
        var methods = ['get', 'put', 'post', 'patch', 'delete', 'options', 'head', 'trace'];
        var operations = [];

        for (var _i2 = 0, _Object$keys2 = Object.keys(openApiResource); _i2 < _Object$keys2.length; _i2++) {
          var key = _Object$keys2[_i2];

          if (methods.includes(key) === false) {
            continue;
          }

          var operation = openApiResource[key];

          if (operation) {
            operations.push(OperationTransformer.execute(spec, operation, key, resourceHeaders));
          }
        }

        resources.push({
          path: path,
          operations: operations,
          name: openApiResource.summary,
          description: openApiResource.description,
          parameters: ParameterTransformer.execute(spec, params)
        });
      }

      return resources;
    }
  }]);

  return ResourceTransformer;
}();

function _await$1(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var OpenApiAdapter = /*#__PURE__*/function () {
  function OpenApiAdapter() {
    _classCallCheck(this, OpenApiAdapter);
  }

  _createClass(OpenApiAdapter, [{
    key: "execute",
    value: function execute(path, config, logger) {
      try {
        return _catch(function () {
          return _await$1(Parser.load(path), function (result) {
            var specification = new Specification(result);
            return {
              info: InformationTransformer.execute(specification),
              auth: AuthenticationTransformer.execute(specification),
              groups: [],
              resources: ResourceTransformer.execute(specification),
              securedBy: specification.entity.security || []
            };
          });
        }, function (error) {
          if (error.code === 'ENOENT') {
            error.message = "".concat(path, ": No such file or directory");
          } else if (error.name !== 'ParsingException') {
            error.message = "".concat(path, " is not a valid OpenAPI schema. \n").concat(error.message);
          }

          throw error;
        });
      } catch (e) {
        return Promise.reject(e);
      }
    }
  }, {
    key: "name",
    value: function name() {
      return 'openapi';
    }
  }]);

  return OpenApiAdapter;
}();

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

module.exports = OpenApiAdapter;
